package eu.etaxonomy.taxeditor;

import org.apache.log4j.Logger;
import org.eclipse.equinox.p2.core.IProvisioningAgent;
import org.eclipse.equinox.p2.engine.IProfileRegistry;
import org.eclipse.equinox.p2.operations.ProvisioningSession;
import org.eclipse.equinox.p2.ui.Policy;
import org.eclipse.equinox.p2.ui.ProvisioningUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.packageadmin.PackageAdmin;

import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * The activator class controls the plug-in life cycle
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public class TaxonomicEditorPlugin extends AbstractUIPlugin {
	private static final Logger logger = Logger
			.getLogger(TaxonomicEditorPlugin.class);

	// The plug-in ID
	/** Constant <code>PLUGIN_ID="eu.etaxonomy.taxeditor.application"</code> */
	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.application";

	// The shared instance
	private static TaxonomicEditorPlugin plugin;

	private PackageAdmin packageAdmin;

	private ServiceReference packageAdminRef;

	/** Constant <code>bundleContext</code> */
	protected static BundleContext bundleContext;


	private ServiceRegistration policyRegistration;

	private ProvisioningSession session;
	private ProvisioningUI ui;

//	private CloudPolicy policy;

	/**
	 * The constructor
	 */
	public TaxonomicEditorPlugin() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	/** {@inheritDoc} */
	@Override
    public void start(BundleContext context) throws Exception {
		super.start(context);

		plugin = this;
		bundleContext = context;

		packageAdminRef = bundleContext.getServiceReference(PackageAdmin.class.getName());
		packageAdmin = (PackageAdmin) bundleContext.getService(packageAdminRef);

		logger.debug("Taxonomic Editor started.");
	}

	private Bundle getBundle(String symbolicName) {
		if (packageAdmin == null) {
            return null;
        }
		Bundle[] bundles = packageAdmin.getBundles(symbolicName, null);
		if (bundles == null) {
            return null;
        }
		// Return the first bundle that is not installed or uninstalled
		for (int i = 0; i < bundles.length; i++) {
			if ((bundles[i].getState() & (Bundle.INSTALLED | Bundle.UNINSTALLED)) == 0) {
				return bundles[i];
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	/** {@inheritDoc} */
	@Override
	public void stop(BundleContext context) throws Exception {
	    plugin = null;
	    super.stop(context);
	    MessagingUtils.info("Taxonomic Editor stopped.");
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static TaxonomicEditorPlugin getDefault() {
	    return plugin;
	}

	public static BundleContext getContext() {
	    return bundleContext;
	}

	public ProvisioningUI getProvisioningUI() {

	    if (ui == null) {
	        ServiceReference reference = bundleContext.getServiceReference(IProvisioningAgent.SERVICE_NAME);
	        IProvisioningAgent agent = (IProvisioningAgent) bundleContext.getService(reference);
	        session = new ProvisioningSession(agent);
	        Policy policy = null;// = (Policy) ServiceHelper.getService(ProvUIActivator.getContext(), Policy.class.getName());
	        if (policy == null) {
	            policy = new Policy();
	        }
	        ui = new ProvisioningUI(session, IProfileRegistry.SELF, policy);
	    }
	    return ui;
	}


}
